#ifndef __ALGORITHMAN__LIST_CPP
    #define __ALGORITHMAN__LIST_CPP

    
    namespace AlgorithMan
    {

        
        // ====================================================================================== Constructor, Destructor

        
        template<class T> list<T>::list()
        {
            count = 0;
            first = new listelement<T>(); // dummy at the beginning of the list
            last = new listelement<T>(); // dummy at the end of the list
            first->next = last;
            first->previous = first;
            last->next = last;
            last->previous = first;
        }
        
        template<class T> list<T>::list(const list<T>& copy)
            : list()
        {
            for(iterator i = copy.begin(); i != copy.end(); i++)
                push_front(*i);
        }

        template<class T> list<T>::list(list<T>&& mov)
        {
            // steal the memory
            count = mov.count;
            first = mov.first;
            last = mov.last;
            // reset mov (so that the destructor does not get confused)
            mov.first = new listelement<T>();
            mov.last = new listelement<T>();
            mov.first->next = mov.last;
            mov.first->previous = mov.first;
            mov.last->previous = mov.first;
            mov.last->next = mov.last;
            mov.count = 0;
        }
        
        template<class T> list<T>::list(int n, T *array)
            : list()
        {
            for(int i = 0; i < n; ++i)
                push_back(array[i]);
        }
        
        template<class T> list<T>::~list()
        {
            clear();
            delete first;
            delete last;
        }
        
        template<class T> void list<T>::clear()
        {
            if(count <= 0)
                return;
            
            listelement<T>* i = first->next;
            do
            {
                i = i->next;
                delete i->previous;
            } while(i != last);
            
            first->next = last;
            last->previous = first;
        }    

        template<class T> int list<T>::size() const
        {
            return count;
        }


        
        // ===================================================================================== traversal
        
        
        
        template<class T> typename list<T>::iterator list<T>::begin() const
        {
            return iterator(first->next);
        }
        
        template<class T> T& list<T>::front() const
        {
            return first->next->content;
        }
        
        template<class T> typename list<T>::iterator list<T>::end() const
        {
            return iterator(last);
        }

        template<class T> T& list<T>::back() const
        {
            return last->previous->content;
        }
        
        template<class T> typename list<T>::iterator list<T>::at(int pos) const
        {
            if(pos < 0 || pos >= count)
                return end();

            list<T>::iterator result;
            if(pos < count/2) // find faster way to pos
            {
                for(result = begin(); pos > 0; --pos)
                    result++;
            }
            else
            {
                pos++;
                for(result = end()-1; pos < count; ++pos)
                    result--;
            }

            return result;
        }
        
        template<class T> typename list<T>::iterator list<T>::find(const T& what) const
        {
            list<T>::iterator result = begin();
            
            for(; result != end(); result++)
                if(*result == what)
                    break;
                    
            return result;
        }
        
        template<class T> typename list<T>::iterator list<T>::find(const list<T>& a) const
        {
            int max = count - a.count;
            listelement<T> *u, *v, *run=first->next;
            for(int i = 0; i <= max; ++i, run=run->next)
            {
                bool found = true;
                for(u=run, v=a.first->next; v != a.last && !found; u=u->next, v=v->next)
                    if(v->content != u->content)
                        found = false;

                if(found)
                    return iterator(run);
            }
            return end();
        }

        
        
        // ====================================================================================== Adding
        
        
        
        template<class T> typename list<T>::iterator list<T>::push_front(const T& what)  
        {
            return insert(first, what);
        }
        
        template<class T> typename list<T>::iterator list<T>::push_back(const T& what)
        {
            return insert(last->previous, what);
        }
        
        template<class T> typename list<T>::iterator list<T>::insertsorted(const T& what)
        {
            listelement<T>* run = first;
            listelement<T>* runnext = first->next;
            while(runnext != last)
            {
                if(what < runnext->content)
                    break;
                run = runnext;
                runnext = runnext->next;
            }
            return insert(iterator(run), what);
        }
        
        template<class T> typename list<T>::iterator list<T>::insert(list<T>::iterator after, const T& what)
        {
            listelement<T>* pos = after.position;
            listelement<T>* bak = pos->next;
            pos->next = new listelement<T>(what);
            pos->next->next = bak;
            pos->next->previous = pos;
            pos->next->next->previous = pos->next;
            ++count;
            return ++after;
        }
        
        template<class T> typename list<T>::iterator list<T>::append(const list<T> &a)
        {
            return insert(last->previous, a);
        }
        
        template<class T> typename list<T>::iterator list<T>::appendanddestroy(list<T> &a)
        {
            return insertanddestroy(iterator(last->previous), a);
        }
        
        template<class T> typename list<T>::iterator list<T>::insert(iterator pos, const list<T> &a)
        {
            for(list<T>::iterator i = a.end()-1; i != a.first; i--)
                insert(pos, i.position->content);
            return ++pos;
        }
        
        template<class T> typename list<T>::iterator list<T>::insertanddestroy(iterator ipos, list<T> &a)
        {
            if(a.count == 0)
                return end();
            listelement<T>* pos = ipos.position;
            
            listelement<T>* bak = pos->next;
            pos->next = a.first->next;
            a.first->next->previous = pos;
            
            a.last->previous->next = bak;
            bak->previous = a.last->previous;

            count += a.count;
            
            // clear a
            a.last->previous = a.first;
            a.first->next = a.last;
            a.count = 0;
            return ++ipos;
        }

        
        
        // ====================================================================================== Deleting
        
        
        
        template<class T> void list<T>::erase_at(int pos)
        {
            if(pos < 0 || pos >= count)
                return;
            erase(at(pos));
        }
        
        template<class T> bool list<T>::erase_content(const T& what)
        {
            list<T>::iterator i = find(what);
            if(i == end())
                return false;
            erase(i);
            return true;
        }
        
        template<class T> void list<T>::erase_all(const T& what)
        {
            iterator n = begin();
            for(iterator i = begin(); i != end(); i = n)
            {
                n++;
                if(*i == what)
                    erase(i);
            }
        }
        
        template<class T> void list<T>::pop_front()
        {
            if(count <= 0) // must not erase dummy elements
                return;
            erase(begin());
        }
        
        template<class T> void list<T>::pop_back()
        {
            if(count <= 0) // must not erase dummy elements
                return;
            erase(iterator(last->previous));
        }
        
        template<class T> void list<T>::erase(iterator pos)
        {
            pos.position->previous->next = pos.position->next;
            pos.position->next->previous = pos.position->previous;
            delete pos.position;
            count--;
        }
        
        
        
        // ====================================================================================== Misc

        
        
        template<class T> bool list<T>::sorted() const
        {
            for(listelement<T> *run = first->next; run->next != last; run = run->next)
                if(run->next->content < run->content)
                    return false;
            return true;
        }
        
        template<class T> void list<T>::swap(typename list<T>::iterator a, typename list<T>::iterator b)
        {
            listelement<T>* A = a.position;
            listelement<T>* B = b.position;
            listelement<T>* bak = A->next;
            A->next = B->next;
            B->next = bak;
            
            A->next->previous = A;
            B->next->previous = B;
            
            bak = A->previous;
            A->previous = B->previous;
            B->previous = bak;
            
            B->previous->next = B;
            A->previous->next = A;
        }
        
        template<class T> list<T> list<T>::split(iterator ipos)
        {
            list<T> result;
            listelement<T>* pos = ipos.position;
            
            result.first->next = pos->next;
            pos->next->previous = result.first;
            pos->next = last;
            result.last->previous = last->previous;
            last->previous->next = result.last;
            last->previous = pos;

            result.count = 0;
            for(list<T>::iterator i = result.begin(); i != result.end(); ++i)
                result.count++;
            
            count = 0;
            for(list<T>::iterator i = begin(); i != end(); ++i)
                count++;
            
            return result;
        }
        
        template<class T> list<T> list<T>::SubListFT(int from, int to)
        {
            list<T> result;
            list<T>::iterator i = at(from);
            listelement<T> *run = *i;
            for(int i = from; i <= to && i < count; ++i, run=run->next)
                result.push_back(run->content);
            return result;
        }
        
        template<class T> list<T> list<T>::SubListFC(int from, int Count)
        {
            return SubListFT(from, from+Count-1);
        }
        
        template<class T> T* list<T>::ToArray()
        {
            T* result = malloc(count * sizeof(T));
            listelement<T> *run = first->next;
            T* resultrun = result;
            for(; run != last; ++resultrun, run=run->next)
                *resultrun = run->content;
            return result;
        }
        
        
        
        // ======================================================================== Sorting
        
        
        
        template<class T> void list<T>::sort(std::ostream& os, int d)
        {
            if(count < 12) // selectionsort (on small lists, primitive sorting is better)
            {
                for(iterator i = begin(); i != end(); i++)
                {
                    iterator minj = i;
                    for(iterator j = i+1; j != end(); j++)
                        if(*j < *minj)
                            minj = j;
                    if(i != minj)
                    {
                        swap(i,minj);
                        i = minj; // otherwise, i would continue at the previous position of minj
                    }
                }
            }
            else // mergesort
            {
                list<T> r = split(at((count-1)/2));
                sort(os,d+1);
                r.sort(os,d+1);
                merge(r);
            }
        }
        
        template<class T> void list<T>::merge(list<T>& r)
        {
            listelement<T>* i = first;
            listelement<T>* bak;
            
            while(i->next != last && r.count > 0)
            {
                if(i->next->content > r.front())
                {
                    insert(iterator(i), r.front());
                    r.pop_front();
                }
                i = i->next;
            }
            
            appendanddestroy(r); // rest of r (if any)            
        }
        
        // ======================================================================== Operators
        
        
        
        template<class T> T& list<T>::operator[](int pos)
        {
            return *at(pos);
        }
        
        template<class T> void list<T>::operator=(const list<T> &a)
        {
            clear();
            append(a);
        }
        
        template<class T> bool list<T>::operator==(const list<T> &a) const
        {
            if(count != a.count)
                return false;
            for(listelement<T>* run=first->next, *arun=a.first->next; run != last; run=run->next, arun=arun->next)
                if(run->content != arun->content)
                    return false;
            return true;
        }
        
        template<class T> bool list<T>::operator!=(const list<T> &a) const
        {
            return !operator==(a);
        }
        
        template<class T> void list<T>::operator += (const list<T> &a)
        {
            append(a);
        }
        
        template<class T> list<T> list<T>::operator + (const list<T> &a) const
        {
            list<T> result;
            result.append(*this);
            result.append(a);
            return result;
        }
        
        template<class T> std::ostream& operator<<(std::ostream& os, const list<T>& a)
        {
            if(a.size() == 0)
                return os;
            typename list<T>::iterator i = a.begin();
            os << *i;
            i++;
            for(; i != a.end(); i++)
                os << ',' << *i;
            return os;
        }
        

    
    
    
    
    
        template<class T> listelement<T>::listelement()
        {
            next = this;
            previous = this;
        }
        
        template<class T> listelement<T>::listelement(const T& content)
            : listelement()
        {
            this->content = content;
        }
        
        template<class T> listelement<T>::~listelement()
        {
        }
    

    
    

    
    
        template<class T> listiterator<T>::listiterator(listelement<T>* position)
        {
            this->position = position;
        }
        
        template<class T> listiterator<T>::~listiterator()
        {
        }


        
        template<class T> bool listiterator<T>::operator==(const listiterator<T>& a)
        {
            return position == a.position;
        }
        
        template<class T> bool listiterator<T>::operator!=(const listiterator<T>& a)
        {
            return position != a.position;
        }
        
        
    
        template<class T> listiterator<T> listiterator<T>::operator++() // prefix increment
        {
            position = position->next;
            return *this;
        }

        template<class T> listiterator<T> listiterator<T>::operator--() // prefix decrement
        {
            position = position->previous;
            return *this;
        }
        
        template<class T> listiterator<T> listiterator<T>::operator++(int) // postfix increment
        {
            position = position->next;
            return listiterator<T>(position->previous);
        }

        template<class T> listiterator<T> listiterator<T>::operator--(int) // postfix decrement
        {
            position = position->previous;
            return listiterator<T>(position->next);
        }


        
        template<class T> listiterator<T> listiterator<T>::operator+=(int i)
        {
            if(i < 0)
                return operator-=(-i);
            for(;i>0;--i)
                operator++();
            return *this;
        }
        
        template<class T> listiterator<T> listiterator<T>::operator-=(int i)
        {
            if(i < 0)
                return operator+=(-i);
            for(;i>0;--i)
                operator--();
            return *this;
        }
        
        template<class T> listiterator<T> listiterator<T>::operator+(int i)
        {
            return listiterator<T>(*this) += i;
        }

        template<class T> listiterator<T> listiterator<T>::operator-(int i)
        {
            return listiterator<T>(*this) -= i;
        }
        
        template<class T> T& listiterator<T>::operator*()
        {
            return position->content;
        }

        template<class T> T* listiterator<T>::operator->()
        {
            return &(position->content);
        }
        
    }
    
#endif
