#ifndef __ALGORITHMAN__LIST_H
    #define __ALGORITHMAN__LIST_H

    #include <iostream>

    namespace AlgorithMan
    {
   
        template<class T> class listiterator;
        template<class T> class listelement;
        
        template<class T> class list
        {
            protected:
                int count;
                listelement<T> *first;
                listelement<T> *last;

                // -------------------------------- System
                //void QuickSort(listElement *l, listElement *r, int L, int R);
            public:
                typedef listiterator<T> iterator;
                
                list();
                list(const list<T>&); // copy constructor
                list(list<T>&&); // move constructor
                list(int n, T* array);
                
                ~list();
                
                void clear();
                int size() const;

                // ------------------------------- traversal
                iterator begin() const;
                T& front() const;
                iterator end() const;
                T& back() const;
                iterator at(int pos) const;
                iterator find(const T& what) const;
                iterator find(const list<T>& a) const;

                // -------------------------------- Adding
                iterator push_front(const T& what);
                iterator push_back(const T& what);
                iterator insertsorted(const T& what);
                iterator insert(iterator pos, const T& what);
                iterator append(const list<T> &a);
                iterator appendanddestroy(list<T> &a);
                iterator insert(iterator pos, const list<T> &a);
                iterator insertanddestroy(iterator pos, list<T> &a);
                
                // ------------------------------ Deleting
                void erase_at(int pos);
                bool erase_content(const T& what);
                void erase_all(const T& what);
                void pop_front();
                void pop_back();
                void erase(iterator pos);
                
                // ---------------------------------- Misc
                bool sorted() const;
                void sort(std::ostream&, int d);
                void merge(list<T>& a);
                void swap(iterator a, iterator b);
                list<T> split(iterator afterpos);
                list<T> SubListFT(int from, int to);
                list<T> SubListFC(int from, int Count);
                T* ToArray();
                
                // ----------------------------- Operators
                T& operator[](int pos);
                void operator =  (const list<T>& a);
                bool operator == (const list<T>& a) const;
                bool operator != (const list<T>& a) const;
                void operator += (const list<T>& a);
                list<T> operator + (const list<T>& a) const;
        };

        
        template<class T> class listiterator
        {
            friend class list<T>;
            protected:
                listelement<T>* position;
                listiterator(listelement<T>* position = NULL);
            public:
                virtual ~listiterator();
                
                bool operator==(const listiterator<T>&);
                bool operator!=(const listiterator<T>&);
                
                listiterator<T> operator++();    // prefix increment
                listiterator<T> operator++(int); // postfix increment
                listiterator<T> operator--();    // prefix decrement
                listiterator<T> operator--(int); // postfix decrement
                
                listiterator<T> operator+(int);
                listiterator<T> operator-(int);
                listiterator<T> operator+=(int);
                listiterator<T> operator-=(int);
                
                T& operator*(); // dereference
                T* operator->();
        };
        
        
        template<class T> class listelement
        {
            friend class list<T>;
            friend class listiterator<T>;
            protected:
                T content;
                listelement *next;
                listelement *previous;
                
                listelement();
                listelement(const T& content);
            public:
                virtual ~listelement();
        };
        
        
        template<class T> std::ostream& operator<<(std::ostream& os, const list<T> &a);
    }

    
    
    #include "list.cpp"
   
#endif
